@@ -1,26 +1,28 @@
-package OpenResty;
+package OpenResty::Handler::Login;
 
 #use Smart::Comments;
 use strict;
 use warnings;
-use vars qw($Cache $UUID $Dumper $Backend);
+
 use CGI::Simple::Cookie;
-use Encode 'is_utf8';
+use OpenResty::Util;
+use Params::Util qw( _STRING );
+use Encode qw( decode is_utf8 );
 
 #*login = \&login_by_sql;
 *login = \&login_by_perl;
 
 sub GET_login_user {
-    my ($self, $bits) = @_;
+    my ($self, $openresty, $bits) = @_;
     my $user = $bits->[1];
-    $self->login($user);
+    $self->login($openresty, $user);
 }
 
 sub GET_login_user_password {
-    my ($self, $bits) = @_;
+    my ($self, $openresty, $bits) = @_;
     my $user = $bits->[1];
     my $password = $bits->[2];
-    $self->login($user, { password => $password });
+    $self->login($openresty, $user, { password => $password });
 }
 
 sub trim_sol {
@@ -33,8 +35,8 @@ sub trim_sol {
 }
 
 sub login_by_sql {
-    my ($self, $user, $params) = @_;
-    _STRING($user) or die "Bad user name: ", $Dumper->($user), "\n";
+    my ($self, $openresty, $user, $params) = @_;
+    _STRING($user) or die "Bad user name: ", $OpenResty::Dumper->($user), "\n";
     $params ||= {};
     ### $params
     ### caller: caller()
@@ -47,10 +49,10 @@ sub login_by_sql {
     } elsif ($user =~ /^\w+$/) {
         $account = $&;
     } else {
-        die "Bad user name: ", $Dumper->($user), "\n";
+        die "Bad user name: ", $OpenResty::Dumper->($user), "\n";
     }
-    _IDENT($account) or die "Bad account name: ", $Dumper->($account), "\n";
-    _IDENT($role) or die "Bad role name: ", $Dumper->($role), "\n";
+    _IDENT($account) or die "Bad account name: ", $OpenResty::Dumper->($account), "\n";
+    _IDENT($role) or die "Bad role name: ", $OpenResty::Dumper->($role), "\n";
     ### $role
     # this part is lame?
     if (!$account) {
@@ -58,15 +60,15 @@ sub login_by_sql {
     }
 
     ### True sol: $true_sol
-    $self->set_user($account);
-    $Backend->login($account, $role, $captcha, $password);
+    $openresty->set_user($account);
+    $OpenResty::Backend->login($account, $role, $captcha, $password);
 
     if (defined $captcha) {
         my ($id, $user_sol) = split /:/, $captcha, 2;
         ### Captcha ID: $id
-        my $true_sol = $Cache->get($id);
+        my $true_sol = $OpenResty::Cache->get($id);
         ### True sol: $true_sol
-        $Cache->remove($id);
+        $OpenResty::Cache->remove($id);
         if (!defined $true_sol) {
             die "Capture ID is bad or expired.\n";
         }
@@ -88,24 +90,24 @@ sub login_by_sql {
         }
     }
 
-    $self->set_role($role);
+    $openresty->set_role($role);
 
-    my $session_from_cookie = $self->{_session_from_cookie};
+    my $session_from_cookie = $openresty->{_session_from_cookie};
     ### Get session ID from cookie: $session_from_cookie
     if ($session_from_cookie) {
         $OpenResty::Cache->remove($session_from_cookie)
     }
 
-    my $captcha_from_cookie = $self->{_captcha_from_cookie};
+    my $captcha_from_cookie = $openresty->{_captcha_from_cookie};
     if ($captcha_from_cookie) {
         $OpenResty::Cache->remove($captcha_from_cookie);
     }
 
-    my $uuid = $UUID->create_str;
-    if ($self->{_use_cookie}) {
-        $self->{_cookie} = { session => $uuid };
+    my $uuid = $OpenResty::UUID->create_str;
+    if ($openresty->{_use_cookie}) {
+        $openresty->{_cookie} = { session => $uuid };
     }
-    $Cache->set($uuid => "$account.$role", 8 * 3600);  # expire in 8 h
+    $OpenResty::Cache->set($uuid => "$account.$role", 8 * 3600);  # expire in 8 h
 
     return {
         success => 1,
@@ -116,8 +118,8 @@ sub login_by_sql {
 }
 
 sub login_by_perl {
-    my ($self, $user, $params) = @_;
-    _STRING($user) or die "Bad user name: ", $Dumper->($user), "\n";
+    my ($self, $openresty, $user, $params) = @_;
+    _STRING($user) or die "Bad user name: ", $OpenResty::Dumper->($user), "\n";
     $params ||= {};
     ### $params
     ### caller: caller()
@@ -130,22 +132,22 @@ sub login_by_perl {
     } elsif ($user =~ /^\w+$/) {
         $account = $&;
     } else {
-        die "Bad user name: ", $Dumper->($user), "\n";
+        die "Bad user name: ", $OpenResty::Dumper->($user), "\n";
     }
-    _IDENT($account) or die "Bad account name: ", $Dumper->($account), "\n";
-    _IDENT($role) or die "Bad role name: ", $Dumper->($role), "\n";
+    _IDENT($account) or die "Bad account name: ", $OpenResty::Dumper->($account), "\n";
+    _IDENT($role) or die "Bad role name: ", $OpenResty::Dumper->($role), "\n";
     ### $role
     # this part is lame?
     if (!$account) {
         die "Login required.\n";
     }
-    if (!$self->has_user($account)) {
+    if (!$openresty->has_user($account)) {
         ### Found user: $user
         die "Account \"$account\" does not exist.\n";
     }
-    $self->set_user($account);
+    $openresty->set_user($account);
 
-    if (!$self->has_role($role)) {
+    if (!$openresty->has_role($role)) {
         ### Found user: $user
         die "Role \"$role\" does not exist.\n";
     }
@@ -159,15 +161,15 @@ sub login_by_perl {
         if (!$id or !$user_sol) {
             die "Bad captcha parameter: $captcha\n";
         }
-        my $res = $self->select("select count(*) from _roles where name = " . Q($role) . " and login = 'captcha'");
+        my $res = $openresty->select("select count(*) from _roles where name = " . Q($role) . " and login = 'captcha'");
         ### with captcha: $res
         if ($res->[0][0] == 0) {
             die "Cannot login as $account.$role via captchas.\n";
         }
         ### Captcha ID: $id
-        my $true_sol = $Cache->get($id);
+        my $true_sol = $OpenResty::Cache->get($id);
         ### True sol: $true_sol
-        $Cache->remove($id);
+        $OpenResty::Cache->remove($id);
         if (!defined $true_sol) {
             die "Capture ID is bad or expired.\n";
         }
@@ -188,13 +190,13 @@ sub login_by_perl {
             die "Solution to the captcha is incorrect.\n";
         }
     } elsif (defined $password) {
-        my $res = $self->select("select count(*) from _roles where name = " . Q($role) . " and login = 'password' and password = " . Q($password) . ";");
+        my $res = $openresty->select("select count(*) from _roles where name = " . Q($role) . " and login = 'password' and password = " . Q($password) . ";");
         ### with password: $res
         if ($res->[0][0] == 0) {
             die "Password for $account.$role is incorrect.\n";
         }
     } else {
-        my $res = $self->select("select count(*) from _roles where name = " . Q($role) . " and login = 'anonymous';");
+        my $res = $openresty->select("select count(*) from _roles where name = " . Q($role) . " and login = 'anonymous';");
         ### no password: $res
         ### no password (2): $res->[0][0]
         if ($res->[0][0] == 0) {
@@ -202,24 +204,24 @@ sub login_by_perl {
             die "Password for $account.$role is required.\n";
         }
     }
-    $self->set_role($role);
+    $openresty->set_role($role);
 
-    my $session_from_cookie = $self->{_session_from_cookie};
+    my $session_from_cookie = $openresty->{_session_from_cookie};
     ### Get session ID from cookie: $session_from_cookie
     if ($session_from_cookie) {
         $OpenResty::Cache->remove($session_from_cookie)
     }
 
-    my $captcha_from_cookie = $self->{_captcha_from_cookie};
+    my $captcha_from_cookie = $openresty->{_captcha_from_cookie};
     if ($captcha_from_cookie) {
         $OpenResty::Cache->remove($captcha_from_cookie);
     }
 
-    my $uuid = $UUID->create_str;
-    if ($self->{_use_cookie}) {
-        $self->{_cookie} = { session => $uuid };
+    my $uuid = $OpenResty::UUID->create_str;
+    if ($openresty->{_use_cookie}) {
+        $openresty->{_cookie} = { session => $uuid };
     }
-    $Cache->set($uuid => "$account.$role", 8 * 3600);  # expire in 8 h
+    $OpenResty::Cache->set($uuid => "$account.$role", 8 * 3600);  # expire in 8 h
 
     return {
         success => 1,
