@@ -1,12 +1,13 @@
-package OpenResty;
+package OpenResty::Handler::Version;
 
 use strict;
 use warnings;
 
 use FindBin;
 use OpenResty;
+use OpenResty::Util;
 
-use vars qw($VERSION $Revision);
+our $Revision;
 
 sub trim {
     (my $s = $_[0]) =~ s/\s+//gs;
@@ -14,7 +15,7 @@ sub trim {
 }
 
 sub GET_version {
-    my ($self, $bits) = @_;
+    my ($self, $openresty, $bits) = @_;
     my $s;
     eval {
             $s = slurp("$FindBin::Bin/../revision")
@@ -29,7 +30,7 @@ sub GET_version {
         }
         $backend .= " ($host)";
     }
-    (my $ver = $VERSION) =~ s/(\d+)\.0*(\d+).0*(\d+)/$1\.$2\.$3/;
+    (my $ver = $OpenResty::VERSION) =~ s/(\d+)\.0*(\d+).0*(\d+)/$1\.$2\.$3/;
     return "OpenResty $ver (revision $Revision) with the $backend backend.\nCopyright (c) 2007-2008 by Yahoo! China EEEE Works, Alibaba Inc.\n";
 }
 
